import datetime

from edge import eventlog
from edge.image.types import ContrastScheduleConf
from edge.model.geolocation import Geolocation
from edge.image.astral import Location


class IContrastSchedule:
    def __init__(self):
        pass

    def test_active(self, im_datetime_utc, im_geolocation):
        """
        :type im_datetime_utc: datetime.datetime
        :type im_geolocation: Geolocation
        :rtype: bool
        """
        raise NotImplementedError


class ContrastSchedule(IContrastSchedule):
    def __init__(self, conf):
        """
        :type conf: ContrastScheduleConf
        """
        IContrastSchedule.__init__(self)
        self._conf = conf

        self._last_geolocation = None
        self._last_datetime = None

        self._twilight_dt_min = None
        self._twilight_dt_max = None

    def _test_geolocation_changed(self, im_geolocation):
        """
        :type im_geolocation: Geolocation | None
        :rtype: bool
        """
        if im_geolocation is None:
            pass

        elif self._last_geolocation is None:
            self._last_geolocation = im_geolocation
            return True

        elif self._last_geolocation != im_geolocation:
            self._last_geolocation = im_geolocation
            return True

        return False

    def _test_date_changed(self, im_datetime):
        """
        :type im_datetime: datetime.datetime
        :rtype: bool
        """
        if self._last_datetime is None:
            self._last_datetime = im_datetime
            return True

        elif self._last_datetime.day != im_datetime.day:
            self._last_datetime = im_datetime
            return True

        return False

    def test_active(self, im_datetime_utc, im_geolocation):
        """
        :type im_datetime_utc: datetime.datetime
        :type im_geolocation: Geolocation | None
        :rtype: bool
        """
        if self._conf.mode == self._conf.MODE_ALWAYS:
            return True

        elif self._conf.mode == self._conf.MODE_NEVER:
            return False

        elif self._conf.mode == self._conf.MODE_TIME:
            # TODO: works only if time_from from this day and time_to from next day
            return self._conf.time_from < im_datetime_utc.time() or im_datetime_utc.time() < self._conf.time_to

        elif self._conf.mode == self._conf.MODE_NIGHT:
            day_changed = self._test_date_changed(im_datetime_utc)
            geo_changed = self._test_geolocation_changed(im_geolocation)

            if day_changed or geo_changed:
                if self._last_geolocation is None:
                    return False

                location = Location(self._last_geolocation.latitude, self._last_geolocation.longitude)
                self._twilight_dt_min = location.sunset(im_datetime_utc)
                self._twilight_dt_max = location.sunrise(im_datetime_utc + datetime.timedelta(days=1))

                eventlog.log_i('day-changed:{}, geo-changed:{}, twilight:({}, {})'.format(
                    day_changed, geo_changed, self._twilight_dt_min.isoformat(), self._twilight_dt_max.isoformat()))

            if None not in (self._twilight_dt_min, self._twilight_dt_max):
                return self._twilight_dt_min < im_datetime_utc < self._twilight_dt_max

        return False
